/*
 * File:   KNA9crossC_PIC12F1572.c
 * Author: shima(JR1KNA)
 * Device: PIC12F1572
 * Created on 2021/10/11, 18:00
 */

#include <xc.h>
//#include <stdlib.h>
#include "Flash.h"
#include "HEFlash.h"
#ifndef _XTAL_FREQ
 // Unless already defined assume 4MHz system frequency
 // This definition is required to calibrate __delay_us() and __delay_ms()
#define _XTAL_FREQ 16000000
#endif

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable (4x PLL disabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOREN = OFF    // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

void init(void)
{
    OSCCON = 0b01111000;        //01111 =16 MHz, 01x = Internal oscillator block
    OPTION_REG = 0b11011111;    // TMR0CS:0 = Internal instruction cycle clock (FOSC/4), PS<2:0>: Prescaler Rate Select bits
    INTCON = 0b11001000;        //bit 7 GIE: Global Interrupt Enable, bit 6 PEIE: Peripheral Interrupt Enable, bit 3 IOCIE: Interrupt-on-Change Enable bit
    PIE1 = 0b00000011;          //bit 1 TMR2IE: TMR2 to PR2 Match Interrupt Enable bit, bit 0 TMR1IE: TMR1 Overflow Interrupt Enable bit
    T1CON = 0b00110101;         //bit 5-4:11 = 1:8 Prescale Value, bit 2:1 = Do not synchronize external clock, bit 0:1 = Enables Timer1
    T2CON = 0b01111111;         //bit 2 TMR2ON: Timer2 On bit, T2CKPS<1:0>: Timer2 Clock Prescale Select bits
    PR2 = 248;
    PIE3 = 0b01000000;          //bit 6 PWM3IE: PWM3 Interrupt Enable bit
    PWM3CON = 0b11100000;       //1 = PWM module is enabled, 1 = Output to PWMx pin is enabled, PWM Module Output Value bit
    PWM3INTE = 0b00000001;      //bit 0 PRIF: Period Interrupt Flag bit
    PWM3CLKCON =0b00000000;     //bit 6-4 PS<2:0>: Clock Source Prescaler Select bits; 16MHz>16MHz 
    IOCAN = 0b00001000;         //IOCAN<5:0>: Interrupt-on-Change PORTA Negative Edge Enable bits
    ANSELA = 0b00000000;        //PORTA ANALOG SELECT REGISTER
    TRISA = 0b00001000;         // 1 = PORTA pin configured as an input (tri-stated), 0 = PORTA pin configured as an output
    PORTA = 0b00101010;         //RA<5:0>: PORTA I/O Value bits
}

unsigned char JR[]={0x45,0x44,0x55,0x23,0x54,0x24,0x53,0x35,0x63,0x45,0x42,0x34,0x33,0x56,0x55,0x54,
0x32,0x33,0x35,0x56,0x54,0x44,0x21,0x34,0x44,0x76,0x45,0x52,0x03,0x43,0x47,0x74,
0x55,0x20,0x24,0x35,0x77,0x45,0x42,0x12,0x33,0x56,0x76,0x54,0x21,0x12,0x44,0x56,
0x75,0x33,0x21,0x14,0x54,0x57,0x52,0x33,0x11,0x45,0x45,0x65,0x33,0x32,0x23,0x54,
0x55,0x54,0x33,0x23,0x34,0x45,0x45,0x53,0x23,0x32,0x2F,0xFF,0xFF,0xFF,0xFF,0xFF};
unsigned char sin11[]={4,5,7,7,6,4,3,1,0,0,2};
unsigned char sin10[]={4,6,7,7,6,4,2,0,0,1};
unsigned char sin09[]={4,6,7,7,5,2,0,0,1};
unsigned char env[]={73,58,40,40,36,29,23,23,14,11,9,9,8,7,6,6};

unsigned char h,i,j,k,l,m,n,t, N1,N2, MOD, pos;
unsigned int A;
bit SET;

void main()
{
    init();
    MOD = FLASH_read(0x7FF);
    if(!MOD){MOD = 1;}
     
    PWM3PRL = 0xFF;       //16MHz>>  0x03FF:15.625kHz;    16MHz>>  0x03E8:16kHz
    PWM3PRH = 0x03;
    PWM3PHL = 0x00;
    PWM3PHH = 0x00;
       
    while(1){
        pos = 2;
        //JR part sampling sound
        while(MOD&0b00000001){
            for(j=0; j<12; j++){
                for(m=0; m<2; m++){
                    for(k=0; k<74; k++){
                        A = env[j] * (JR[k]>>4);
                        PWM3DCL = (A);
                        PWM3DCH = (A>>8);
                        PWM3LDCON = 0b10000000;
                        
                        while(!SET){}
                        SET = 0; 
                        
                        A = env[j] * (JR[k]&0b00001111);
                        PWM3DCL = (A);
                        PWM3DCH = (A>>8);
                        PWM3LDCON = 0b10000000;
                        
                        while(!SET){}
                        SET = 0;
                    }
                }
            }
        }
        //JR synthesizer sound
        N1 = 11;     //=T1/Ts=Fs/f1  (7.8125KHz/710Hz)
        N2 = 10;     //=T2/Ts=Fs/f12  (7.8125KHz/760Hz) 
        while(MOD&0b00000010){
            for(j=0; j<16; j++){
                for(k=0; k<N1*2; k++){
                    for(l=0; l<N2; l++){
                        if(m==N1){m = 0;}
                        A = env[j] * (sin10[l] + sin11[m]);
                        ++m;
                        PWM3DCL = (A);
                        PWM3DCH = (A>>8);
                        PWM3LDCON = 0b10000000;
                        
                        while(!SET){}
                        SET = 0;
                    }
                }    
            }
        }
        //Odakyu synthesizer sound
        pos = 3;
        N1 = 11;     //=T1/Ts=Fs/f1  (5.208KHz/470Hz)
        N2 = 9;      //=T2/Ts=Fs/f12  (5.208KHz/570Hz)
        while(MOD&0b00000100){
            for(j=0; j<14; j++){
                for(k=0; k<N1*2; k++){
                    for(l=0; l<N2; l++){
                        if(m==N1){m = 0;}
                        A = env[j] * (sin09[l] + sin11[m]);
                        ++m;
                        PWM3DCL = (A);
                        PWM3DCH = (A>>8);
                        PWM3LDCON = 0b10000000;
                                 
                        while(!SET){}
                        SET = 0;
                    }
                }
            }
        }
    }
}    
    
void interrupt isr(void)
{   
    if(PWM3INTF & 0b00000001){
        ++t;
        if(t==pos){
            SET = 1;
            t = 0;
        }
        PWM3INTF = 0;
    }
    else if(TMR1IF){
        ++i;
        if(i==5){
            RA4 = ~RA4;
            RA5 = ~RA5;
            i=0;
        }
        TMR1IF = 0;
    }
    else if(TMR2IF){
        ++h;
        if(h==10){
            RA0 = ~RA0;
            RA1 = ~RA1;
            h=0;
        }
        TMR2IF = 0;
    }
    else if(IOCAF3){
        MOD <<= 1;
        if(MOD>0b00001111){MOD = 0b00000001;}
        FLASH_erase(0x7FF);
        FLASH_write(0x7FF,MOD,0);
        RA4 = ~RA5;
        RA0 = ~RA1;
        IOCAF3 = 0;
    }
}

